@@ -7,6 +7,8 @@
 using LCompilers::ASR::is_a;
 using LCompilers::ASR::down_cast;
 
+Allocator al(8);
+
 namespace LCompilers {
 
 enum Precedence {
@@ -143,15 +145,20 @@ public:
         }
     }
 
-    void handle_line_truncation(std::string &r, int i_level, int line_length=80) {
-        int line_segments_count = r.size()/line_length;
-        for (int i = 1; i <= line_segments_count; i ++) {
-            int index = r.find_last_of(',', line_length*i);
-            r.insert(index + 2, "&\n" + indent +
-                std::string(i_level*indent_spaces, ' '));
+    void handle_line_truncation(std::string &r, int i_level, int line_length=120) {
+        size_t current_pos = 0;
+        std::string indent = std::string(i_level * indent_spaces, ' ');
+        while (current_pos + line_length < r.length()) {
+            size_t break_pos = r.find_last_of(',', current_pos + line_length);
+            if (break_pos == std::string::npos || break_pos <= current_pos) {
+                break_pos = current_pos + line_length - 1;
+            }
+            r.insert(break_pos + 1, "&\n" + indent);
+            current_pos = break_pos + 2 + i_level * indent_spaces;
         }
     }
 
+
     std::string get_type(const ASR::ttype_t *t) {
         std::string r = "";
         switch (t->type) {
@@ -170,8 +177,8 @@ public:
                 r += std::to_string(down_cast<ASR::Complex_t>(t)->m_kind);
                 r += ")";
                 break;
-            } case ASR::ttypeType::Character: {
-                ASR::Character_t *c = down_cast<ASR::Character_t>(t);
+            } case ASR::ttypeType::String: {
+                ASR::String_t *c = down_cast<ASR::String_t>(t);
                 r = "character(len=";
                 if(c->m_len > 0) {
                     r += std::to_string(c->m_len);
@@ -227,8 +234,8 @@ public:
             } case ASR::ttypeType::Pointer: {
                 r = get_type(down_cast<ASR::Pointer_t>(t)->m_type) + ", pointer";
                 break;
-            } case ASR::ttypeType::Struct: {
-                ASR::Struct_t* struct_type = down_cast<ASR::Struct_t>(t);
+            } case ASR::ttypeType::StructType: {
+                ASR::StructType_t* struct_type = down_cast<ASR::StructType_t>(t);
                 std::string struct_name = ASRUtils::symbol_name(struct_type->m_derived_type);
                 r = "type(";
                 r += struct_name;
@@ -239,6 +246,9 @@ public:
                     import_struct_type.push_back(struct_name);
                 }
                 break;
+            } case ASR::ttypeType::CPtr: {
+                r = "type(c_ptr)";
+                break;
             }
             default:
                 throw LCompilersException("The type `"
@@ -296,6 +306,7 @@ public:
         r = "program";
         r += " ";
         r.append(x.m_name);
+        handle_line_truncation(r, 2);
         r += "\n";
         for (auto &item : x.m_symtab->get_scope()) {
             if (is_a<ASR::ExternalSymbol_t>(*item.second)) {
@@ -307,8 +318,8 @@ public:
         r += "\n";
         std::map<std::string, std::vector<std::string>> struct_dep_graph;
         for (auto &item : x.m_symtab->get_scope()) {
-            if (ASR::is_a<ASR::StructType_t>(*item.second) ||
-                    ASR::is_a<ASR::EnumType_t>(*item.second) ||
+            if (ASR::is_a<ASR::Struct_t>(*item.second) ||
+                    ASR::is_a<ASR::Enum_t>(*item.second) ||
                     ASR::is_a<ASR::UnionType_t>(*item.second)) {
                 std::vector<std::string> struct_deps_vec;
                 std::pair<char**, size_t> struct_deps_ptr = ASRUtils::symbol_dependencies(item.second);
@@ -370,6 +381,7 @@ public:
         r = "module";
         r += " ";
         r.append(x.m_name);
+        handle_line_truncation(r, 2);
         r += "\n";
         for (auto &item : x.m_symtab->get_scope()) {
             if (is_a<ASR::ExternalSymbol_t>(*item.second)) {
@@ -388,8 +400,8 @@ public:
         }
         std::map<std::string, std::vector<std::string>> struct_dep_graph;
         for (auto &item : x.m_symtab->get_scope()) {
-            if (ASR::is_a<ASR::StructType_t>(*item.second) ||
-                    ASR::is_a<ASR::EnumType_t>(*item.second) ||
+            if (ASR::is_a<ASR::Struct_t>(*item.second) ||
+                    ASR::is_a<ASR::Enum_t>(*item.second) ||
                     ASR::is_a<ASR::UnionType_t>(*item.second)) {
                 std::vector<std::string> struct_deps_vec;
                 std::pair<char**, size_t> struct_deps_ptr = ASRUtils::symbol_dependencies(item.second);
@@ -507,6 +519,7 @@ public:
                 r += " result(" + return_var + ")";
             }
         }
+        handle_line_truncation(r, 2);
         r += "\n";
 
         inc_indent();
@@ -530,6 +543,7 @@ public:
                 if (i < import_struct_type.size() - 1) {
                     r += ", ";
                 } else {
+                    handle_line_truncation(r, 2);
                     r += "\n";
                 }
             }
@@ -548,6 +562,7 @@ public:
                     inc_indent();
                     visit_symbol(*item.second);
                     r += src;
+                    handle_line_truncation(r, 2);
                     r += "\n";
                     dec_indent();
                     r += indent;
@@ -578,6 +593,7 @@ public:
         std::string r = indent;
         r += "interface ";
         r.append(x.m_name);
+        handle_line_truncation(r, 2);
         r += "\n";
         inc_indent();
         r += indent;
@@ -587,6 +603,7 @@ public:
             if (i < x.n_procs-1) r += ", ";
         }
         dec_indent();
+        handle_line_truncation(r, 2);
         r += "\n";
         r += "end interface ";
         r.append(x.m_name);
@@ -599,7 +616,7 @@ public:
     void visit_ExternalSymbol(const ASR::ExternalSymbol_t &x) {
         ASR::symbol_t *sym = down_cast<ASR::symbol_t>(
             ASRUtils::symbol_parent_symtab(x.m_external)->asr_owner);
-        if (!is_a<ASR::StructType_t>(*sym)) {
+        if (!is_a<ASR::Struct_t>(*sym)) {
             src = indent;
             src += "use ";
             src.append(x.m_module_name);
@@ -609,10 +626,11 @@ public:
         }
     }
 
-    void visit_StructType(const ASR::StructType_t &x) {
+    void visit_Struct(const ASR::Struct_t &x) {
         std::string r = indent;
         r += "type :: ";
         r.append(x.m_name);
+        handle_line_truncation(r, 2);
         r += "\n";
         inc_indent();
         std::vector<std::string> var_order = ASRUtils::determine_variable_declaration_order(x.m_symtab);
@@ -630,7 +648,7 @@ public:
         src = r;
     }
 
-    // void visit_EnumType(const ASR::EnumType_t &x) {}
+    // void visit_Enum(const ASR::Enum_t &x) {}
 
     // void visit_UnionType(const ASR::UnionType_t &x) {}
 
@@ -687,11 +705,12 @@ public:
             visit_expr(*x.m_symbolic_value);
             r += src;
         }
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
 
-    // void visit_ClassType(const ASR::ClassType_t &x) {}
+    // void visit_Class(const ASR::Class_t &x) {}
 
     // void visit_ClassProcedure(const ASR::ClassProcedure_t &x) {}
 
@@ -721,7 +740,9 @@ public:
             }
             if (i < x.n_args-1) r += ", ";
         }
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         src = r;
     }
 
@@ -736,6 +757,7 @@ public:
         r += "to";
         r += " ";
         r += x.m_variable;
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -747,6 +769,7 @@ public:
         r += " = ";
         visit_expr(*x.m_value);
         r += src;
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -776,6 +799,7 @@ public:
             if (i < x.n_vars-1) r += ", ";
         }
         r += ")";
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -816,6 +840,7 @@ public:
             visit_expr(*x.m_head.m_increment);
             r += src;
         }
+        handle_line_truncation(r, 2);
         r += "\n";
         visit_body(x, r);
         r += indent;
@@ -848,6 +873,7 @@ public:
         r += "go to";
         r += " ";
         r += std::to_string(x.m_target_id);
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -857,6 +883,7 @@ public:
         r += std::to_string(x.m_id);
         r += " ";
         r += "continue";
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -869,15 +896,18 @@ public:
         r += src;
         r += ") ";
         r += "then";
+        handle_line_truncation(r, 2);
         r += "\n";
         visit_body(x, r);
-        for (size_t i = 0; i < x.n_orelse; i++) {
+        if (x.n_orelse > 0) {
             r += indent;
             r += "else";
             r += "\n";
             inc_indent();
-            visit_stmt(*x.m_orelse[i]);
-            r += src;
+            for (size_t i = 0; i < x.n_orelse; i++) {
+                visit_stmt(*x.m_orelse[i]);
+                r += src;
+            }
             dec_indent();
         }
         r += indent;
@@ -892,22 +922,33 @@ public:
         std::string r = indent;
         r += "print";
         r += " ";
-        if (x.n_values > 0 && is_a<ASR::StringFormat_t>(*x.m_values[0])) {
-            ASR::StringFormat_t *sf = down_cast<ASR::StringFormat_t>(x.m_values[0]);
-            visit_expr(*sf->m_fmt);
-            if (is_a<ASR::StringConstant_t>(*sf->m_fmt)
-                    && (!startswith(src, "\"(") || !endswith(src, ")\""))) {
-                src = "\"(" + src.substr(1, src.size()-2) + ")\"";
+        if (is_a<ASR::StringFormat_t>(*x.m_text)) {
+            ASR::StringFormat_t *sf = down_cast<ASR::StringFormat_t>(x.m_text);
+            if(sf->m_fmt){
+                visit_expr(*(sf->m_fmt));
+                if (is_a<ASR::StringConstant_t>(*sf->m_fmt)
+                        && (!startswith(src, "\"(") || !endswith(src, ")\""))) {
+                    src = "\"(" + src.substr(1, src.size()-2) + ")\"";
+                }
+                r += src;
+            } else {
+                r += "*";
             }
+            for (size_t i = 0; i < sf->n_args; i++) {
+                r += ", ";
+                visit_expr(*sf->m_args[i]);
+                r += src;
+            }
+        } else if (ASR::is_a<ASR::String_t>(*ASRUtils::expr_type(x.m_text))) {
+            r += "*";
+            r += ", ";
+            visit_expr(*x.m_text);
             r += src;
         } else {
-            r += "*";
-        }
-        for (size_t i = 0; i < x.n_values; i++) {
-            r += ", ";
-            visit_expr(*x.m_values[i]);
-            r += src;
+            throw CodeGenError("print statment supported for stringformat and single character argument",
+                x.base.base.loc);
         }
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -942,6 +983,7 @@ public:
             r += src;
         }
         r += ")";
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -958,6 +1000,7 @@ public:
             throw CodeGenError("close() function must be called with a file unit number");
         }
         r += ")";
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -1005,6 +1048,7 @@ public:
             r += src;
             if (i < x.n_values - 1) r += ", ";
         }
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -1024,12 +1068,16 @@ public:
         }
         if (x.n_values > 0 && is_a<ASR::StringFormat_t>(*x.m_values[0])) {
             ASR::StringFormat_t *sf = down_cast<ASR::StringFormat_t>(x.m_values[0]);
-            visit_expr(*sf->m_fmt);
-            if (is_a<ASR::StringConstant_t>(*sf->m_fmt)
-                    && (!startswith(src, "\"(") || !endswith(src, ")\""))) {
-                src = "\"(" + src.substr(1, src.size()-2) + ")\"";
+            if(sf->m_fmt){
+                visit_expr(*sf->m_fmt);
+                if (is_a<ASR::StringConstant_t>(*sf->m_fmt)
+                        && (!startswith(src, "\"(") || !endswith(src, ")\""))) {
+                    src = "\"(" + src.substr(1, src.size()-2) + ")\"";
+                }
+                r += src;
+            } else {
+                r += "*";
             }
-            r += src;
         } else {
             r += "*";
         }
@@ -1039,6 +1087,7 @@ public:
             r += src;
             if (i < x.n_values-1) r += ", ";
         }
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -1046,6 +1095,7 @@ public:
     void visit_Return(const ASR::Return_t &/*x*/) {
         std::string r = indent;
         r += "return";
+        handle_line_truncation(r, 2);
         r += "\n";
         src = r;
     }
@@ -1056,7 +1106,9 @@ public:
         r += " (";
         visit_expr(*x.m_test);
         r += src;
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         inc_indent();
         if (x.n_body > 0) {
             for(size_t i = 0; i < x.n_body; i ++) {
@@ -1099,8 +1151,10 @@ public:
             r += src;
             if (i < x.n_args-1) r += ", ";
         }
-        r += ")\n";
-        handle_line_truncation(r, 1);
+        
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         src = r;
     }
 
@@ -1112,11 +1166,14 @@ public:
         r += "(";
         visit_expr(*x.m_test);
         r += src;
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         visit_body(x, r);
         for (size_t i = 0; i < x.n_orelse; i++) {
             r += indent;
             r += "else where";
+            handle_line_truncation(r, 2);
             r += "\n";
             inc_indent();
             visit_stmt(*x.m_orelse[i]);
@@ -1139,7 +1196,9 @@ public:
         r += " (";
         visit_expr(*x.m_test);
         r += src;
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         visit_body(x, r);
         r += indent;
         r += "end do";
@@ -1205,38 +1264,23 @@ public:
         src = r;
     }
 
-    void visit_TypeInquiry(const ASR::TypeInquiry_t &x) {
-        std::string out = "";
-        switch (x.m_inquiry_id) {
-            SET_INTRINSIC_NAME(Epsilon,   "epsilon"  );
-            SET_INTRINSIC_NAME(Huge,      "huge"     );
-            SET_INTRINSIC_NAME(Precision, "precision");
-            SET_INTRINSIC_NAME(Radix,     "radix"    );
-            SET_INTRINSIC_NAME(Range,     "range"    );
-            SET_INTRINSIC_NAME(Rank,      "rank"     );
-            SET_INTRINSIC_NAME(Tiny,      "tiny"     );
-            SET_INTRINSIC_NAME(BitSize,   "bit_size" );
-            SET_INTRINSIC_NAME(NewLine,   "new_line" );
-            SET_INTRINSIC_NAME(Kind,      "kind"     );
-            default : {
-                throw LCompilersException("TypeInquiry: `"
-                    + ASRUtils::get_intrinsic_name(x.m_inquiry_id)
-                    + "` is not implemented");
-            }
-        }
-        this->visit_expr(*x.m_arg);
-        out += "(" + src + ")";
-        src = out;
-    }
-
     void visit_IntrinsicImpureSubroutine( const ASR::IntrinsicImpureSubroutine_t &x ) {
         std::string out;
         out = "call ";
-        switch ( x.m_intrinsic_id ) {
+        switch ( x.m_sub_intrinsic_id ) {
             SET_INTRINSIC_SUBROUTINE_NAME(RandomNumber, "random_number");
+            SET_INTRINSIC_SUBROUTINE_NAME(RandomInit, "random_init");
+            SET_INTRINSIC_SUBROUTINE_NAME(RandomSeed, "random_seed");
+            SET_INTRINSIC_SUBROUTINE_NAME(GetCommand, "get_command");
+            SET_INTRINSIC_SUBROUTINE_NAME(GetCommandArgument, "get_command_argument");
+            SET_INTRINSIC_SUBROUTINE_NAME(GetEnvironmentVariable, "get_environment_variable");
+            SET_INTRINSIC_SUBROUTINE_NAME(ExecuteCommandLine, "execute_command_line");
+            SET_INTRINSIC_SUBROUTINE_NAME(Srand, "srand");
+            SET_INTRINSIC_SUBROUTINE_NAME(SystemClock, "system_clock");
+            SET_INTRINSIC_SUBROUTINE_NAME(DateAndTime, "date_and_time");
             default : {
                 throw LCompilersException("IntrinsicImpureSubroutine: `"
-                    + ASRUtils::get_intrinsic_name(x.m_intrinsic_id)
+                    + ASRUtils::get_intrinsic_name(x.m_sub_intrinsic_id)
                     + "` is not implemented");
             }
         }
@@ -1250,135 +1294,7 @@ public:
         src = out;
     }
 
-    void visit_IntrinsicElementalFunction(const ASR::IntrinsicElementalFunction_t &x) {
-        std::string out;
-        switch (x.m_intrinsic_id) {
-            SET_INTRINSIC_NAME(Abs, "abs");
-            SET_INTRINSIC_NAME(Exp, "exp");
-            SET_INTRINSIC_NAME(Max, "max");
-            SET_INTRINSIC_NAME(Min, "min");
-            SET_INTRINSIC_NAME(Sqrt, "sqrt");
-            SET_INTRINSIC_NAME(Mod, "mod");
-            SET_INTRINSIC_NAME(Sin, "sin");
-            SET_INTRINSIC_NAME(Cos, "cos");
-            SET_INTRINSIC_NAME(Tan, "tan");
-            SET_INTRINSIC_NAME(Sind, "sind");
-            SET_INTRINSIC_NAME(Cosd, "cosd");
-            SET_INTRINSIC_NAME(Tand, "tand");
-            SET_INTRINSIC_NAME(Asin, "asin");
-            SET_INTRINSIC_NAME(Acos, "acos");
-            SET_INTRINSIC_NAME(Atan, "atan");
-            SET_INTRINSIC_NAME(Asind, "asind");
-            SET_INTRINSIC_NAME(Acosd, "acosd");
-            SET_INTRINSIC_NAME(Atand, "atand");
-            SET_INTRINSIC_NAME(Sinh, "sinh");
-            SET_INTRINSIC_NAME(Cosh, "cosh");
-            SET_INTRINSIC_NAME(Tanh, "tanh");
-            SET_INTRINSIC_NAME(Asinh, "asinh");
-            SET_INTRINSIC_NAME(Acosh, "acosh");
-            SET_INTRINSIC_NAME(Atanh, "atanh");
-            SET_INTRINSIC_NAME(Char, "char");
-            SET_INTRINSIC_NAME(StringContainsSet, "verify");
-            SET_INTRINSIC_NAME(StringFindSet, "scan");
-            SET_INTRINSIC_NAME(SubstrIndex, "index");
-            SET_INTRINSIC_NAME(Modulo, "modulo");
-            SET_INTRINSIC_NAME(Poppar, "poppar");
-            SET_INTRINSIC_NAME(Merge, "merge");
-            SET_INTRINSIC_NAME(Aimag, "aimag");
-            SET_INTRINSIC_NAME(SelectedIntKind, "selected_int_kind");
-            SET_INTRINSIC_NAME(SelectedRealKind, "selected_real_kind");
-            SET_INTRINSIC_NAME(SelectedCharKind, "selected_char_kind");
-            SET_INTRINSIC_NAME(Bgt, "bgt");
-            SET_INTRINSIC_NAME(Blt, "blt");
-            SET_INTRINSIC_NAME(Bge, "bge");
-            SET_INTRINSIC_NAME(Ble, "ble");
-            SET_INTRINSIC_NAME(Lgt, "lgt");
-            SET_INTRINSIC_NAME(Llt, "llt");
-            SET_INTRINSIC_NAME(Lge, "lge");
-            SET_INTRINSIC_NAME(Lle, "lle");
-            SET_INTRINSIC_NAME(Iand, "iand");
-            SET_INTRINSIC_NAME(Ior, "ior");
-            SET_INTRINSIC_NAME(Ieor, "ieor");
-            SET_INTRINSIC_NAME(Ibclr, "ibclr");
-            SET_INTRINSIC_NAME(Ibset, "ibset");
-            SET_INTRINSIC_NAME(Btest, "btest");
-            SET_INTRINSIC_NAME(Ibits, "ibits");
-            SET_INTRINSIC_NAME(Not, "not");
-            SET_INTRINSIC_NAME(Digits, "digits");
-            SET_INTRINSIC_NAME(Rrspacing, "rrspacing");
-            SET_INTRINSIC_NAME(Repeat, "repeat");
-            SET_INTRINSIC_NAME(Adjustl, "adjustl");
-            SET_INTRINSIC_NAME(Adjustr, "adjustr");
-            SET_INTRINSIC_NAME(Ichar, "ichar");
-            SET_INTRINSIC_NAME(MaxExponent, "maxexponent");
-            SET_INTRINSIC_NAME(MinExponent, "minexponent");
-            SET_INTRINSIC_NAME(Shiftr, "shiftr");
-            SET_INTRINSIC_NAME(Rshift, "rshift");
-            SET_INTRINSIC_NAME(Shiftl, "shiftl");
-            SET_INTRINSIC_NAME(Dshiftl, "dshiftl");
-            SET_INTRINSIC_NAME(Ishft, "ishft");
-            SET_INTRINSIC_NAME(Ishftc, "ishftc");
-            SET_INTRINSIC_NAME(Floor, "floor");
-            SET_INTRINSIC_NAME(Ceiling, "ceiling");
-            SET_INTRINSIC_NAME(Maskr, "maskr");
-            SET_INTRINSIC_NAME(Maskl, "maskl");
-            SET_INTRINSIC_NAME(Sngl, "sngl");
-            SET_INTRINSIC_NAME(Log, "log");
-            SET_INTRINSIC_NAME(Log10, "log10");
-            SET_INTRINSIC_NAME(Isnan, "isnan");
-            SET_INTRINSIC_NAME(LogGamma, "log_gamma");
-            SET_INTRINSIC_NAME(Erf, "erf");
-            SET_INTRINSIC_NAME(Erfc, "erfc");
-            SET_INTRINSIC_NAME(Gamma, "gamma");
-            SET_INTRINSIC_NAME(BesselJ0, "bessel_j0");
-            SET_INTRINSIC_NAME(BesselJ1, "bessel_j1");
-            SET_INTRINSIC_NAME(BesselY0, "bessel_y0");
-            SET_INTRINSIC_NAME(BesselY1, "bessel_y1");
-            SET_INTRINSIC_NAME(Atan2, "atan2");
-            SET_INTRINSIC_NAME(Dreal, "dreal");
-            SET_INTRINSIC_NAME(Exp2, "exp2");
-            SET_INTRINSIC_NAME(Expm1, "expm1");
-            SET_INTRINSIC_NAME(FMA, "fma");
-            SET_INTRINSIC_NAME(FloorDiv, "floordiv");
-            SET_INTRINSIC_NAME(Trailz, "trailz");
-            SET_INTRINSIC_NAME(Nearest, "nearest");
-            SET_INTRINSIC_NAME(Spacing, "spacing");
-            SET_INTRINSIC_NAME(BesselJN, "bessel_jn");
-            SET_INTRINSIC_NAME(BesselYN, "bessel_yn");
-            SET_INTRINSIC_NAME(Mergebits, "merge_bits");
-            SET_INTRINSIC_NAME(Exponent, "exponent");
-            SET_INTRINSIC_NAME(Fraction, "fraction");
-            SET_INTRINSIC_NAME(SetExponent, "set_exponent");
-            SET_INTRINSIC_NAME(Leadz, "leadz");
-            SET_INTRINSIC_NAME(Hypot, "hypot");
-            SET_INTRINSIC_NAME(Scale, "scale");
-            SET_INTRINSIC_NAME(Dprod, "dprod");
-            SET_INTRINSIC_NAME(Sign, "sign");
-            SET_INTRINSIC_NAME(Aint, "aint");
-            SET_INTRINSIC_NAME(Popcnt, "popcnt");
-            SET_INTRINSIC_NAME(Nint, "nint");
-            SET_INTRINSIC_NAME(Anint, "anint");
-            SET_INTRINSIC_NAME(Dim, "dim");
-            SET_INTRINSIC_NAME(Ifix, "ifix");
-            SET_INTRINSIC_NAME(Idint, "idint");
-            SET_INTRINSIC_NAME(Conjg, "conjg");
-            default : {
-                throw LCompilersException("IntrinsicElementalFunction: `"
-                    + ASRUtils::get_intrinsic_name(x.m_intrinsic_id)
-                    + "` is not implemented");
-            }
-        }
-        out += "(";
-        for (size_t i = 0; i < x.n_args; i ++) {
-            visit_expr(*x.m_args[i]);
-            out += src;
-            if (i < x.n_args-1) out += ", ";
-        }
-        out += ")";
-        src = out;
-    }
-
-    void setArrIntrinsicName(std::string &out, std::string func_name, const ASR::IntrinsicArrayFunction_t &x) {
+    void visit_IntrinsicElementalFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicElementalFunction_t &x) {
         out += func_name;
         visit_expr(*x.m_args[0]);
         out += "(" + src;
@@ -1389,137 +1305,73 @@ public:
         }
         out += ")";
         src = out;
-        out = "";
     }
 
-    void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t &x) {
+    void visit_IntrinsicElementalFunction(const ASR::IntrinsicElementalFunction_t &x) {
         std::string out;
-        switch (x.m_arr_intrinsic_id) {
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Any)) :
-                setArrIntrinsicName(out, "any", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::All)) :
-                setArrIntrinsicName(out, "all", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Iany)) :
-                setArrIntrinsicName(out, "iany", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Iall)) :
-                setArrIntrinsicName(out, "iall", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Sum)) :
-                setArrIntrinsicName(out, "sum", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Product)) :
-                setArrIntrinsicName(out, "product", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Shape)) :
-                setArrIntrinsicName(out, "shape", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::MaxVal)) :
-                setArrIntrinsicName(out, "maxval", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::MinVal)) :
-                setArrIntrinsicName(out, "minval", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Norm2)) :
-                setArrIntrinsicName(out, "norm2", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Transpose)) :
-                setArrIntrinsicName(out, "transpose", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Pack)) :
-                setArrIntrinsicName(out, "pack", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Unpack)) :
-                setArrIntrinsicName(out, "unpack", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Iparity)) :
-                setArrIntrinsicName(out, "iparity", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Count)) :
-                setArrIntrinsicName(out, "count", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::Parity)) :
-                setArrIntrinsicName(out, "parity", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::DotProduct)) :
-                setArrIntrinsicName(out, "dot_product", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::MatMul)) :
-                setArrIntrinsicName(out, "matmul", x);
-                break;
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::MaxLoc)) : {
-                out += "maxloc";
-                visit_expr(*x.m_args[0]);
-                out += "(" + src;
-                if (x.n_args > 1) {
-                    out += ", ";
-                    visit_expr(*x.m_args[1]);
-                    out += src;
-                    if (x.n_args > 2) {
-                        out += ", ";
-                        visit_expr(*x.m_args[2]);
-                        out += src;
-                        if (x.n_args > 3) {
-                            out += ", ";
-                            visit_expr(*x.m_args[3]);
-                            out += src;
-                            if (x.n_args > 4) {
-                                out += ", ";
-                                visit_expr(*x.m_args[4]);
-                                out += src;
-                            }
-                        }
-                    }
-                }
-                out += ")";
-                src = out;
-                out = "";
-                break;
-            }
-            case (static_cast<int64_t>(ASRUtils::IntrinsicArrayFunctions::MinLoc)) : {
-                out += "minloc";
-                visit_expr(*x.m_args[0]);
-                out += "(" + src;
-                if (x.n_args > 1) {
-                    out += ", ";
-                    visit_expr(*x.m_args[1]);
-                    out += src;
-                    if (x.n_args > 2) {
-                        out += ", ";
-                        visit_expr(*x.m_args[2]);
-                        out += src;
-                        if (x.n_args > 3) {
-                            out += ", ";
-                            visit_expr(*x.m_args[3]);
-                            out += src;
-                            if (x.n_args > 4) {
-                                out += ", ";
-                                visit_expr(*x.m_args[4]);
-                                out += src;
-                            }
-                        }
-                    }
-                }
-                out += ")";
-                src = out;
-                out = "";
-                break;
-            }
-            default : {
-                throw LCompilersException("IntrinsicArrayFunction: `"
-                    + ASRUtils::get_array_intrinsic_name(x.m_arr_intrinsic_id)
-                    + "` is not implemented");
-            }
-        }
+        std::string intrinsic_func_name = ASRUtils::get_intrinsic_name(static_cast<int64_t>(x.m_intrinsic_id));
+        if(intrinsic_func_name == "StringFindSet") intrinsic_func_name = "scan";
+        else if(intrinsic_func_name == "StringContainsSet") intrinsic_func_name = "verify";
+        else if(intrinsic_func_name == "SubstrIndex") intrinsic_func_name = "index";
+        else if(intrinsic_func_name == "SelectedRealKind") intrinsic_func_name = "selected_real_kind";
+        else if(intrinsic_func_name == "SelectedIntKind") intrinsic_func_name = "selected_int_kind";
+        else if(intrinsic_func_name == "SelectedCharKind") intrinsic_func_name = "selected_char_kind";
+        else if(intrinsic_func_name == "LogGamma") intrinsic_func_name = "log_gamma";
+        else if(intrinsic_func_name == "SetExponent") intrinsic_func_name = "set_exponent";
+        else if(intrinsic_func_name == "Mergebits") intrinsic_func_name = "merge_bits";
+        else if(intrinsic_func_name == "StringLenTrim") intrinsic_func_name = "len_trim";
+        else if(intrinsic_func_name == "StringTrim") intrinsic_func_name = "trim";
+        visit_IntrinsicElementalFunction_helper(out, intrinsic_func_name, x);
+    }
+
+    void visit_TypeInquiry_helper(std::string &out, std::string func_name, const ASR::TypeInquiry_t &x) {
+        out += func_name;
+        visit_expr(*x.m_arg);
         out += "(" + src + ")";
         src = out;
     }
 
+    void visit_TypeInquiry(const ASR::TypeInquiry_t &x) {
+        std::string out;
+        std::string intrinsic_func_name = ASRUtils::get_intrinsic_name(static_cast<int64_t>(x.m_inquiry_id));
+        if(intrinsic_func_name == "BitSize") intrinsic_func_name = "bit_size";
+        else if(intrinsic_func_name == "NewLine") intrinsic_func_name = "new_line";
+        visit_TypeInquiry_helper(out, intrinsic_func_name, x);
+    }
+
+    void visit_IntrinsicArrayFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicArrayFunction_t &x) {
+        out += func_name;
+        visit_expr(*x.m_args[0]);
+        out += "(" + src;
+        for (size_t i = 1; i < x.n_args; i++) {
+            out += ", ";
+            visit_expr(*x.m_args[i]);
+            out += src;
+        }
+        out += ")";
+        src = out;
+    }
+
+    void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t &x) {
+        std::string out;
+        std::string intrinsic_func_name = ASRUtils::get_array_intrinsic_name(static_cast<int64_t>(x.m_arr_intrinsic_id));
+        if(intrinsic_func_name == "DotProduct") intrinsic_func_name = "dot_product";
+        if(intrinsic_func_name == "Spread" && x.m_overload_id == -1){
+            ASR::ArrayPhysicalCast_t *arr_physical = ASR::down_cast<ASR::ArrayPhysicalCast_t>(x.m_args[0]);
+            if(ASR::is_a<ASR::ArrayConstant_t>(*arr_physical->m_arg)){
+                ASR::ArrayConstant_t *arr_const = ASR::down_cast<ASR::ArrayConstant_t>(arr_physical->m_arg);
+                x.m_args[0] = ASRUtils::fetch_ArrayConstant_value(al, arr_const, 0);
+            } else if(ASR::is_a<ASR::ArrayConstructor_t>(*arr_physical->m_arg)){
+                ASR::ArrayConstructor_t *arr_const = ASR::down_cast<ASR::ArrayConstructor_t>(arr_physical->m_arg);
+                x.m_args[0] = arr_const->m_args[0];
+            }           
+        }
+        visit_IntrinsicArrayFunction_helper(out, intrinsic_func_name, x);
+    }
+
     // void visit_IntrinsicImpureFunction(const ASR::IntrinsicImpureFunction_t &x) {}
 
-    void visit_StructTypeConstructor(const ASR::StructTypeConstructor_t &x) {
+    void visit_StructConstructor(const ASR::StructConstructor_t &x) {
         std::string r = indent;
         r += ASRUtils::symbol_name(x.m_dt_sym);
         r += "(";
@@ -1532,7 +1384,7 @@ public:
         src = r;
     }
 
-    // void visit_EnumTypeConstructor(const ASR::EnumTypeConstructor_t &x) {}
+    // void visit_EnumConstructor(const ASR::EnumConstructor_t &x) {}
 
     // void visit_UnionTypeConstructor(const ASR::UnionTypeConstructor_t &x) {}
 
@@ -1545,6 +1397,7 @@ public:
     }
 
     void visit_IntegerConstant(const ASR::IntegerConstant_t &x) {
+        // TODO: handle IntegerBOZ
         src = std::to_string(x.m_n);
         int kind = ASRUtils::extract_kind_from_ttype_t(x.m_type);
         if (kind != 4) {
@@ -1555,8 +1408,6 @@ public:
         last_expr_precedence = Precedence::Ext;
     }
 
-    // void visit_IntegerBOZ(const ASR::IntegerBOZ_t &x) {}
-
     // void visit_IntegerBitNot(const ASR::IntegerBitNot_t &x) {}
 
     void visit_IntegerUnaryMinus(const ASR::IntegerUnaryMinus_t &x) {
@@ -1598,7 +1449,7 @@ public:
     void visit_RealConstant(const ASR::RealConstant_t &x) {
         int kind = ASRUtils::extract_kind_from_ttype_t(x.m_type);
         if (kind >= 8) {
-            src = ASRUtils::to_string_with_precision(x.m_r, 16) + "d0";
+            src = ASRUtils::to_string_with_precision(x.m_r, 16) + "_8";
         } else {
             src = ASRUtils::to_string_with_precision(x.m_r, 8);
         }
@@ -1758,6 +1609,10 @@ public:
         src = r;
     }
 
+    void visit_StringPhysicalCast(const ASR::StringPhysicalCast_t &x) {
+        visit_expr(*x.m_arg);
+    }
+
     // void visit_CPtrCompare(const ASR::CPtrCompare_t &x) {}
 
     // void visit_SymbolicCompare(const ASR::SymbolicCompare_t &x) {}
@@ -2016,7 +1871,9 @@ public:
             r += src;
             if (i < x.n_test-1) r += ", ";
         }
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         inc_indent();
         for(size_t i = 0; i < x.n_body; i ++) {
             visit_stmt(*x.m_body[i]);
@@ -2038,7 +1895,9 @@ public:
             visit_expr(*x.m_end);
             r += src;
         }
-        r += ")\n";
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
         inc_indent();
         for(size_t i = 0; i < x.n_body; i ++) {
             visit_stmt(*x.m_body[i]);
