@@ -4,6 +4,7 @@
 #include <memory>
 
 #include <llvm/IR/Value.h>
+#include <llvm/IR/Module.h>
 #include <llvm/IR/IRBuilder.h>
 #include <libasr/asr.h>
 
@@ -19,10 +20,6 @@
 
 namespace LCompilers {
 
-    #define get_builder0() llvm::BasicBlock &entry_block = builder->GetInsertBlock()->getParent()->getEntryBlock(); \
-        llvm::IRBuilder<> builder0(context); \
-        builder0.SetInsertPoint(&entry_block, entry_block.getFirstInsertionPt()); \
-
     // Platform dependent fast unique hash:
     static inline uint64_t get_hash(ASR::asr_t *node)
     {
@@ -74,7 +71,7 @@ namespace LCompilers {
         llvm::Function *fn_printf = module.getFunction("_lfortran_printf");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lfortran_printf", &module);
         }
@@ -87,22 +84,70 @@ namespace LCompilers {
         llvm::Function *fn_printf = module.getFunction("_lcompilers_string_format_fortran");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getInt8PtrTy(context),
+                    llvm::Type::getInt8Ty(context)->getPointerTo(),
                     {llvm::Type::getInt32Ty(context),
-                    llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lcompilers_string_format_fortran", &module);
         }
         return builder.CreateCall(fn_printf, args);
     }
 
+    static inline llvm::Value* lfortran_str_copy(llvm::Value* dest, llvm::Value *src, bool is_allocatable,
+        llvm::Module &module, llvm::IRBuilder<> &builder, llvm::LLVMContext &context, llvm::Type* string_descriptor ) {
+        if(!is_allocatable){ 
+            std::string runtime_func_name = "_lfortran_strcpy_pointer_string";
+            llvm::Function *fn = module.getFunction(runtime_func_name);
+            if (!fn) {
+                llvm::FunctionType *function_type = llvm::FunctionType::get(
+                        llvm::Type::getVoidTy(context),
+                        {
+                            llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(), 
+                            llvm::Type::getInt8Ty(context)->getPointerTo()
+                        }, false);
+                fn = llvm::Function::Create(function_type,
+                        llvm::Function::ExternalLinkage, runtime_func_name, module);
+            }
+            return builder.CreateCall(fn, {dest, src});
+        } else {
+            std::string runtime_func_name = "_lfortran_strcpy_descriptor_string";
+            llvm::Value *src_char_ptr, *dest_char_ptr, *string_size, *string_capacity;
+            std::vector<llvm::Value*> idx {
+                llvm::ConstantInt::get(context, llvm::APInt(32, 0)),
+                llvm::ConstantInt::get(context, llvm::APInt(32, 0))};
+            // Fetch char* from `src` and `dest` + Fetch string_size, string_capacity from `dest`
+            dest_char_ptr = builder.CreateGEP(string_descriptor, dest, idx);
+            src_char_ptr = builder.CreateLoad(llvm::Type::getInt8Ty(context)->getPointerTo(),
+                builder.CreateGEP(string_descriptor, src, idx));
+            idx[1] = llvm::ConstantInt::get(context, llvm::APInt(32, 1));
+            string_size = builder.CreateGEP(string_descriptor, dest, idx);
+            idx[1] = llvm::ConstantInt::get(context, llvm::APInt(32, 2));
+            string_capacity =  builder.CreateGEP(string_descriptor, dest, idx);
+            llvm::Function *fn = module.getFunction(runtime_func_name);
+            if (!fn) {
+                llvm::FunctionType *function_type = llvm::FunctionType::get(
+                        llvm::Type::getVoidTy(context),
+                        {
+                            llvm::Type::getInt8Ty(context)->getPointerTo()->getPointerTo(), 
+                            llvm::Type::getInt8Ty(context)->getPointerTo(),
+                            llvm::Type::getInt64Ty(context)->getPointerTo(),
+                            llvm::Type::getInt64Ty(context)->getPointerTo()
+                        }, false);
+                fn = llvm::Function::Create(function_type,
+                        llvm::Function::ExternalLinkage, runtime_func_name, module);
+            }
+            return builder.CreateCall(fn, {dest_char_ptr, src_char_ptr, string_size, string_capacity});
+        }
+        
+    }
+
     static inline void print_error(llvm::LLVMContext &context, llvm::Module &module,
         llvm::IRBuilder<> &builder, const std::vector<llvm::Value*> &args)
     {
         llvm::Function *fn_printf = module.getFunction("_lcompilers_print_error");
         if (!fn_printf) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
-                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8PtrTy(context)}, true);
+                    llvm::Type::getVoidTy(context), {llvm::Type::getInt8Ty(context)->getPointerTo()}, true);
             fn_printf = llvm::Function::Create(function_type,
                     llvm::Function::ExternalLinkage, "_lcompilers_print_error", &module);
         }
@@ -134,7 +179,7 @@ namespace LCompilers {
         if (!fn) {
             llvm::FunctionType *function_type = llvm::FunctionType::get(
                 llvm::Type::getVoidTy(context), {
-                    llvm::Type::getInt8PtrTy(context),
+                    llvm::Type::getInt8Ty(context)->getPointerTo(),
                     llvm::Type::getInt1Ty(context)
                 }, true);
             fn = llvm::Function::Create(function_type,
@@ -146,10 +191,7 @@ namespace LCompilers {
 
     namespace LLVM {
 
-        llvm::Value* CreateLoad(llvm::IRBuilder<> &builder, llvm::Value *x);
         llvm::Value* CreateStore(llvm::IRBuilder<> &builder, llvm::Value *x, llvm::Value *y);
-        llvm::Value* CreateGEP(llvm::IRBuilder<> &builder, llvm::Value *x, std::vector<llvm::Value *> &idx);
-        llvm::Value* CreateInBoundsGEP(llvm::IRBuilder<> &builder, llvm::Value *x, std::vector<llvm::Value *> &idx);
         llvm::Value* lfortran_malloc(llvm::LLVMContext &context, llvm::Module &module,
                 llvm::IRBuilder<> &builder, llvm::Value* arg_size);
         llvm::Value* lfortran_realloc(llvm::LLVMContext &context, llvm::Module &module,
@@ -161,8 +203,8 @@ namespace LCompilers {
         static inline bool is_llvm_struct(ASR::ttype_t* asr_type) {
             return ASR::is_a<ASR::Tuple_t>(*asr_type) ||
                    ASR::is_a<ASR::List_t>(*asr_type) ||
-                   ASR::is_a<ASR::Struct_t>(*asr_type) ||
-                   ASR::is_a<ASR::Class_t>(*asr_type)||
+                   ASR::is_a<ASR::StructType_t>(*asr_type) ||
+                   ASR::is_a<ASR::ClassType_t>(*asr_type)||
                    ASR::is_a<ASR::Dict_t>(*asr_type);
         }
         static inline bool is_llvm_pointer(const ASR::ttype_t& asr_type) {
@@ -200,6 +242,7 @@ namespace LCompilers {
             std::map<std::string, std::map<std::string, int>>& name2memidx;
             std::unordered_map<std::uint32_t, std::unordered_map<std::string, llvm::Type*>>& arr_arg_type_cache;
             std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type;
+            std::map<llvm::Value *, llvm::Type *> &ptr_type;
 
             LLVMDictInterface* dict_api_lp;
             LLVMDictInterface* dict_api_sc;
@@ -211,6 +254,7 @@ namespace LCompilers {
             llvm::StructType *complex_type_4, *complex_type_8;
             llvm::StructType *complex_type_4_ptr, *complex_type_8_ptr;
             llvm::PointerType *character_type;
+            llvm::Type* string_descriptor;
 
             LLVMUtils(llvm::LLVMContext& context,
                 llvm::IRBuilder<>* _builder, std::string& der_type_name_,
@@ -221,16 +265,43 @@ namespace LCompilers {
                 std::map<std::string, std::map<std::string, int>>& name2memidx_,
                 CompilerOptions &compiler_options_,
                 std::unordered_map<std::uint32_t, std::unordered_map<std::string, llvm::Type*>>& arr_arg_type_cache_,
-                std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type_);
+                std::map<std::string, std::pair<llvm::Type*, llvm::Type*>>& fname2arg_type_,
+                std::map<llvm::Value *, llvm::Type *> &ptr_type_);
 
             llvm::Value* create_gep(llvm::Value* ds, int idx);
 
+            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, int idx);
+
             llvm::Value* create_gep(llvm::Value* ds, llvm::Value* idx);
 
+            llvm::Value* create_gep2(llvm::Type *t, llvm::Value* ds, llvm::Value* idx);
+
             llvm::Value* create_ptr_gep(llvm::Value* ptr, int idx);
 
+            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, int idx);
+
             llvm::Value* create_ptr_gep(llvm::Value* ptr, llvm::Value* idx);
 
+            llvm::Value* create_ptr_gep2(llvm::Type* type, llvm::Value* ptr, llvm::Value* idx);
+
+            llvm::Value* CreateLoad(llvm::Value *x);
+            llvm::Value* CreateLoad2(llvm::Type *t, llvm::Value *x);
+            llvm::Value* CreateLoad2(ASR::ttype_t *type, llvm::Value *x);
+            llvm::Value* CreateGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateGEP2(llvm::Type *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateGEP2(ASR::ttype_t *type, llvm::Value *x, int idx);
+            llvm::Value* CreateInBoundsGEP(llvm::Value *x, std::vector<llvm::Value *> &idx);
+            llvm::Value* CreateInBoundsGEP2(llvm::Type *t, llvm::Value *x,
+                std::vector<llvm::Value *> &idx);
+
+            llvm::AllocaInst* CreateAlloca(llvm::Type* type,
+                llvm::Value* size=nullptr, std::string Name="",
+                bool is_llvm_ptr=false);
+            llvm::AllocaInst* CreateAlloca(llvm::IRBuilder<> &builder,
+                llvm::Type* type, llvm::Value* size=nullptr, std::string Name="",
+                bool is_llvm_ptr=false);
+
             llvm::Type* getIntType(int a_kind, bool get_pointer=false);
 
             void start_new_block(llvm::BasicBlock *bb);
@@ -250,9 +321,9 @@ namespace LCompilers {
             llvm::Type* getMemberType(ASR::ttype_t* mem_type,
                 ASR::Variable_t* member, llvm::Module* module);
 
-            void createStructContext(ASR::StructType_t* der_type);
+            void createStructTypeContext(ASR::Struct_t* der_type);
 
-            llvm::Type* getStructType(ASR::StructType_t* der_type, llvm::Module* module, bool is_pointer=false);
+            llvm::Type* getStructType(ASR::Struct_t* der_type, llvm::Module* module, bool is_pointer=false);
 
             llvm::Type* getStructType(ASR::ttype_t* _type, llvm::Module* module, bool is_pointer=false);
 
@@ -262,9 +333,9 @@ namespace LCompilers {
             llvm::Type* getUnionType(ASR::ttype_t* _type,
                 llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getClassType(ASR::ClassType_t* der_type, bool is_pointer=false);
+            llvm::Type* getClassType(ASR::Class_t* der_type, bool is_pointer=false);
 
-            llvm::Type* getClassType(ASR::StructType_t* der_type, bool is_pointer=false);
+            llvm::Type* getClassType(ASR::Struct_t* der_type, bool is_pointer=false);
 
             llvm::Type* getClassType(ASR::ttype_t* _type, bool is_pointer=false);
 
@@ -282,6 +353,10 @@ namespace LCompilers {
 
             std::vector<llvm::Type*> convert_args(const ASR::Function_t &x, llvm::Module* module);
 
+            llvm::FunctionType* get_function_type(ASR::FunctionType_t* x, llvm::Module* module);
+
+            std::vector<llvm::Type*> convert_args(ASR::FunctionType_t* x, llvm::Module* module);
+
             llvm::Type* get_type_from_ttype_t(ASR::ttype_t* asr_type,
                 ASR::symbol_t *type_declaration, ASR::storage_typeType m_storage,
                 bool& is_array_type, bool& is_malloc_array_type, bool& is_list,
@@ -456,7 +531,7 @@ namespace LCompilers {
 
             llvm::LLVMContext& context;
             LLVMUtils* llvm_utils;
-            llvm::IRBuilder<>* builder;
+            // llvm::IRBuilder<>* builder;
 
             std::map<std::string, std::pair<llvm::Type*, size_t>> typecode2tupletype;
 
