@@ -207,6 +207,7 @@ public:
         // Avoid inlining if function call accepts a callback argument
         for( size_t i = 0; i < x->n_args; i++ ) {
             if( x->m_args[i].m_value &&
+                ASRUtils::expr_type(x->m_args[i].m_value) &&
                 ASR::is_a<ASR::FunctionType_t>(
                     *ASRUtils::type_get_past_pointer(
                         ASRUtils::expr_type(x->m_args[i].m_value))) ) {
@@ -243,7 +244,9 @@ public:
         if( ASRUtils::is_intrinsic_function2(func) ||
                 std::string(func->m_name) == current_routine ||
                 // Never Inline BindC Function
-                ASRUtils::get_FunctionType(func)->m_abi == ASR::abiType::BindC) {
+                ASRUtils::get_FunctionType(func)->m_abi == ASR::abiType::BindC || 
+                // Never Inline Interface Function
+                ASRUtils::get_FunctionType(func)->m_deftype == ASR::deftypeType::Interface) {
             return ;
         }
 
@@ -313,8 +316,8 @@ public:
             if( !ASR::is_a<ASR::Variable_t>(*itr.second) ||
                  ASRUtils::is_character(*ASRUtils::symbol_type(itr.second)) ||
                  ASRUtils::is_array(ASRUtils::symbol_type(itr.second)) ||
-                 ASR::is_a<ASR::Struct_t>(*ASRUtils::symbol_type(itr.second)) ||
-                 ASR::is_a<ASR::Class_t>(*ASRUtils::symbol_type(itr.second)) ) {
+                 ASR::is_a<ASR::StructType_t>(*ASRUtils::symbol_type(itr.second)) ||
+                 ASR::is_a<ASR::ClassType_t>(*ASRUtils::symbol_type(itr.second)) ) {
                 arg2value.clear();
                 return ;
             }
@@ -335,10 +338,6 @@ public:
                     break;
                 }
                 ASR::ttype_t* local_var_type = func_var->m_type;
-                if( ASR::is_a<ASR::Const_t>(*local_var_type) ) {
-                    local_var_type = ASR::down_cast<ASR::Const_t>(local_var_type)->m_type;
-                }
-                LCOMPILERS_ASSERT(!ASR::is_a<ASR::Const_t>(*local_var_type));
                 ASR::symbol_t* local_var = (ASR::symbol_t*) ASR::make_Variable_t(
                         al, func_var->base.base.loc, current_scope,
                         s2c(al, local_var_name), nullptr, 0, ASR::intentType::Local,
@@ -529,7 +528,7 @@ public:
         pass_result.n = 0;
     }
 
-    void visit_Character(const ASR::Character_t& /*x*/) {
+    void visit_String(const ASR::String_t& /*x*/) {
 
     }
 
